/*
 *   Copyright (c) 2003 Nikodemus Siivola
 *   
 *   Permission is hereby granted, free of charge, to any person obtaining
 *   a copy of this software and associated documentation files (the
 *   "Software"), to deal in the Software without restriction, including
 *   without limitation the rights to use, copy, modify, merge, publish,
 *   distribute, sublicense, and/or sell copies of the Software, and to
 *   permit persons to whom the Software is furnished to do so, subject to
 *   the following conditions:
 *   
 *   The above copyright notice and this permission notice shall be included
 *   in all copies or substantial portions of the Software.
 *   
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 *   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 *   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <unistd.h>
#include <termios.h>
#include <stdlib.h>

typedef struct {
  struct termios *attr;
  void *next;
}
state_t;

state_t * stack = NULL;

int
save_term (void)
{
  state_t * state = malloc (sizeof (state_t));
  struct termios * attr = malloc (sizeof (struct termios));
  if (! (state && attr))
    return 0;

  tcgetattr (STDIN_FILENO, attr);

  state->attr = attr;
  state->next = stack;
  stack = state;

  return 1;
}
  
void
restore_term (void)
{
  state_t * tmp = stack;
  if (!stack) return;
  
  tcsetattr (STDIN_FILENO, TCSANOW, stack->attr);
  stack = stack->next;

  free (tmp->attr);
  free (tmp);
}

int
no_echo (void)
{
  struct termios attr;  
  if (! (isatty (STDIN_FILENO) && save_term ())) 
    return 0;

  tcgetattr (STDIN_FILENO, &attr);
  attr.c_lflag &= ~ECHO;
  tcsetattr (STDIN_FILENO, TCSAFLUSH, &attr);

  return 1;
}
